import * as THREE from '../build/three.module.js';
import Stats from './jsm/libs/stats.module.js';

class App {
    constructor() {
        this._initialize();
    }

    _initialize() {
        this.domWebGL = document.createElement('div');
        document.body.appendChild(this.domWebGL);

        let renderer = new THREE.WebGLRenderer({ antialias: true });
        this.domWebGL.appendChild(renderer.domElement);
        this._renderer = renderer;

        const scene = new THREE.Scene();
        scene.background = new THREE.Color(0x050505);
        this._scene = scene;

        (new THREE.FontLoader()).load("fonts/helvetiker_bold.typeface.json", (font) => {
            this._setupCamera();
            this._setupModel(font);
            this._setupGUI();
            
            window.onresize = this.resize.bind(this);
    
            this.resize();

            this.render();
        });
    }

    _setupCamera() {
        const camera = new THREE.PerspectiveCamera(30, window.innerWidth / window.innerHeight, 1, 10000);
        camera.position.z = 400;
        this._camera = camera;
    }

    _createShaderMaterial() {
        const uniforms = {
            "amplitude": { value: 10.0 },
            "opacity": { value: 0.3 },
            "color": { value: new THREE.Color(0xffffff) },
        };

        const vertexCode = `
            uniform float amplitude;

            attribute vec3 displacement;
            attribute vec3 customColor; 
            
            varying vec3 vColor;

            void main() {
                vec3 newPosition = position + amplitude * displacement;
                vColor = customColor;
                gl_Position = projectionMatrix * modelViewMatrix * vec4(newPosition, 1.0);
            }
        `;

        const fragmentCode = `
            uniform vec3 color;
            uniform float opacity;

            varying vec3 vColor;
            
            void main() {
                gl_FragColor = vec4(vColor * color, opacity);
            }
        `;
        
        const shaderMaterial = new THREE.ShaderMaterial({
            uniforms: uniforms,
            vertexShader: vertexCode,
            fragmentShader: fragmentCode,
            blending: THREE.AdditiveBlending,
            depthTest: false,
            transparent: true
        });

        return { material: shaderMaterial, uniforms: uniforms };
    }

    _setupModel(font) {
        const geometry = new THREE.TextGeometry('Dip2K', {
            font: font,
            size: 50,
            height: 1,
            curveSegments: 20,
            bevelThickness: 2,
            bevelSize: 2,
            bevelEnabled: true,
            bevelSegments: 10
        });

        geometry.center();

        const count = geometry.attributes.position.count;
        const displacement = new THREE.Float32BufferAttribute(count*3, 3);
        geometry.setAttribute("displacement", displacement);

        const customColor = new THREE.Float32BufferAttribute(count*3, 3);
        geometry.setAttribute("customColor", customColor);

        const color = new THREE.Color(0xffffff);

        for(let i=0, l=customColor.count; i<l; i++) {
            color.setHSL(i/l, 0.5, 0.5);
            color.toArray(customColor.array, i*customColor.itemSize);
        }

        const material = this._createShaderMaterial();
        const line = new THREE.Line(geometry, material.material);
        line.rotation.x = 0.2;
        this._scene.add(line);

        this._line = line;
        this._uniforms = material.uniforms;
    }

    update() {
        const time = Date.now() * 0.001;

        //this._line.rotation.y = 0.25 * time;

        this._uniforms.amplitude.value = Math.sin(0.5 * time);
        this._uniforms.color.value.offsetHSL(0.0005, 0, 0);

        const attrubutes = this._line.geometry.attributes;
        const array = attrubutes.displacement.array;

        for(let i=0, l=array.length; i<l; i+=3) {
            array[i] += 1.3 * (0.5 - Math.random());
            array[i+1] += 1.3 * (0.5 - Math.random());
            array[i+2] += 1.3 * (0.5 - Math.random());
        }

        attrubutes.displacement.needsUpdate = true;
    }

    render() {
        requestAnimationFrame(this.render.bind(this));
        
        this.update();
        this._renderer.render(this._scene, this._camera);
        this._stats.update();
    }

    resize() {
        this._camera.aspect = window.innerWidth / window.innerHeight;
        this._camera.updateProjectionMatrix();

        this._renderer.setPixelRatio(window.devicePixelRatio);
        this._renderer.setSize(window.innerWidth, window.innerHeight);
    }

    _setupGUI() {
        const stats = new Stats();
        this.domWebGL.appendChild(stats.dom);
        this._stats = stats;
    }
}

window.onload = function () {
    (new App());//.render();
}